'''Basic test suite for wyvern.py'''

import warnings
from wyvern import *
import unittest
from configparser import ConfigParser

class WyvernTester(unittest.TestCase):
	@classmethod
	def setUpClass(self):
		config = ConfigParser()
		config.read('config.txt')

		# Wyvern clients with read, write, and direct message access
		self.rwdm1 = Wyvern(config.get('Twitter', 'consumer_key' + str(0)),
		                    config.get('Twitter', 'consumer_secret' + str(0)),
		                    config.get('Twitter', 'access_token' + str(0)),
	    	                config.get('Twitter', 'access_token_secret' + str(0)))
		self.rwdm2 = Wyvern(config.get('Twitter', 'consumer_key' + str(1)),
	    	                config.get('Twitter', 'consumer_secret' + str(1)),
	    	                config.get('Twitter', 'access_token' + str(1)),
	        	            config.get('Twitter', 'access_token_secret' + str(1)))

		# Clients with read and write access
		self.rw1 = Wyvern(config.get('Twitter', 'consumer_key' + str(2)),
	    	              config.get('Twitter', 'consumer_secret' + str(2)),
	        	          config.get('Twitter', 'access_token' + str(2)),
	            	      config.get('Twitter', 'access_token_secret' + str(2)))
		self.rw2 = Wyvern(config.get('Twitter', 'consumer_key' + str(3)),
	    	              config.get('Twitter', 'consumer_secret' + str(3)),
	    	              config.get('Twitter', 'access_token' + str(3)),
	    	              config.get('Twitter', 'access_token_secret' + str(3)))

		# Clients with only read access
		self.r1 = Wyvern(config.get('Twitter', 'consumer_key' + str(4)),
	    	             config.get('Twitter', 'consumer_secret' + str(4)),
	        	         config.get('Twitter', 'access_token' + str(4)),
	            	     config.get('Twitter', 'access_token_secret' + str(4)))
		self.r2 = Wyvern(config.get('Twitter', 'consumer_key' + str(5)),
	    	             config.get('Twitter', 'consumer_secret' + str(5)),
	        	         config.get('Twitter', 'access_token' + str(5)),
	            	     config.get('Twitter', 'access_token_secret' + str(5)))

	def test_tweets(self):
		r1 = self.rw1.tweet(status='Testing testing, 1 2 3', trim_user=True)
		r2 = self.rw2.show_tweet(id=r1['id'], trim_user=True)
		r3 = self.r1.show_tweet(id=r1['id'], trim_user=True)
		self.assertEqual(r2, r3)
		self.assertEqual(r1, r2)
		self.assertEqual(r1, r2)

		r4 = self.rw1.destroy_tweet(id=r1['id'])

		r5 = self.rw2.tweet(status='Testing testing, 1 2 3', 
		                    trim_user=True)
		r6 = self.rw1.retweet(id=r5['id'], trim_user=True)

		r7 = self.r2.show_tweet(id=r5['id'], trim_user=True)
		r8 = self.rw2.retweets_of_me(count=1, trim_user=True)
		r9 = self.r1.retweets_of_tweet(id=r5['id'], count=1, 
		                               trim_user=True)
		del r5['retweet_count']
		del r8[0]['retweet_count']
		self.assertEqual(r5, r8[0])
		self.assertEqual(r7, r9[0]['retweeted_status'])
		self.assertEqual(r7['retweet_count'], 1)

		r10 = self.rw2.destroy_tweet(id=r5['id'])

	def test_search(self):
		r1 = self.r1.search_tweets(q='Python 3', count=100)
		self.assertEqual(len(r1['statuses']), 100)

		r2 = self.r2.search_tweets(q='Python 3')
		self.assertEqual(len(r2['statuses']), 15)

		r3 = self.r1.search_users(q='Python 3', count=20)
		self.assertEqual(len(r3), 20)

	def test_direct_messages(self):
		name1 = self.rwdm1.my_account_settings()['screen_name']
		name2 = self.rwdm2.my_account_settings()['screen_name']
		r1 = self.rwdm1.follow(screen_name=name2)
		r2 = self.rwdm2.follow(screen_name=name1)

		r3 = self.rwdm1.send_direct_message(screen_name=name2, text='Hey you')
		r4 = self.rwdm1.my_sent_direct_messages(count=1)
		self.assertEqual(r3, r4[0])

		r5 = self.rwdm2.my_received_direct_messages(count=1)
		self.assertEqual(r3['text'], r5[0]['text'])

		r6 = self.rwdm2.destroy_direct_message(id=r4[0]['id'])
		r7 = self.rwdm1.unfollow(screen_name=name2)
		r8 = self.rwdm2.unfollow(screen_name=name1)

	def test_friends_and_followers(self):
		name1 = self.rwdm1.my_account_settings()['screen_name']
		name2 = self.rwdm2.my_account_settings()['screen_name']
		name3 = self.rw1.my_account_settings()['screen_name']
		name4 = self.rw2.my_account_settings()['screen_name']

		r1 = self.rwdm1.follow(screen_name=name2)
		r2 = self.rwdm2.follow(screen_name=name1)
		r3 = self.rw1.follow(screen_name=name1)
		r4 = self.r1.show_friendship(source_screen_name=name1,
		                             target_screen_name=name2)
		r5 = self.r1.show_friendship(source_screen_name=name1,
		                             target_screen_name=name3)
		r6 = self.r1.show_friendship(source_screen_name=name3,
		                             target_screen_name=name4)

		self.assertEqual(r4['relationship']['target']['following'],
		                 r4['relationship']['source']['followed_by'])
		self.assertEqual(r4['relationship']['target']['followed_by'],
		                 r4['relationship']['source']['following'])
		self.assertEqual(r5['relationship']['source']['following'], False)
		self.assertEqual(r5['relationship']['source']['followed_by'], True)
		self.assertEqual(r6['relationship']['source']['following'],
		                 r6['relationship']['target']['followed_by'])
		self.assertEqual(r6['relationship']['source']['followed_by'],
		                 r6['relationship']['target']['following'])

		r7 = self.rwdm1.unfollow(screen_name=name2)
		r8 = self.rwdm2.unfollow(screen_name=name1)
		r9 = self.rw1.unfollow(screen_name=name1)

	def test_errors(self):
		self.assertRaises(OAuthError, Wyvern, 'a','b','c','d')
		self.assertRaises(OAuthError, self.r1.tweet, status='yeah')
		self.assertRaises(WyvernUserError, self.rw1.retweet, id=0)
		self.assertRaises(WyvernUserError, self.rw2.destroy_tweet, 
		                  id=303191270434684929)
		self.assertRaises(WyvernUserError, self.r1.search_tweets)
		self.assertRaises(OAuthError, self.r1.send_direct_message,
		                  text="I am a rebel", screen_name="danxmoran")
		self.assertRaises(WyvernUserError, self.rwdm1.send_direct_message,
		                  text="Dear Anonymous")
		
		def rate_limit_gen(w):
			while True:
				r = w.rw1.show_tweet(id=303603809391869952)

		self.assertRaises(WyvernUserRateLimitError, rate_limit_gen, self)

if __name__ == '__main__':
	unittest.main()
